package DE_EPAGES::Search::t::Search;
use base DE_EPAGES::Database::API::Test;
use Test::More tests => 28;
use strict;
use DE_EPAGES::Object::API::Factory qw (LoadClassByAlias LoadObjectByPath);
use DE_EPAGES::Object::API::Language qw (GetPKeyLanguageByCode);
use DE_EPAGES::Search::API::Object::PagerSearch;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Search::API::SearchDefinition qw (GetSearchDefinition);

my $ROOTPATH = '/';

sub test {

    my $Object = LoadObjectByPath('/');
    my %Search = (
        'ObjectID' => $Object->id,
        'SearchDefinition' => GetSearchDefinition(LoadClassByAlias('Object'), 'LocalizedText'),
        'Parameters' => {
            'String'   => '%all%',
            'LanguageID' => GetPKeyLanguageByCode('de'),
            'Class.Alias' => 'TestObject',
        },
        'SortParameters'=> [
            'Class.Alias',
            'Alias'
        ],
    );

    my $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 3, 'found correct count');

    $Search{'Parameters'}->{'String'} = '%ALL%';
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 3, 'found correct count (case insensitive)');

    $Search{'Parameters'}->{'LanguageID'} = GetPKeyLanguageByCode('en');
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 0, 'found correct count (english)');

    $Search{'Parameters'}->{'String'} = '%TagList%';
    $Search{'Parameters'}->{'LanguageID'} = GetPKeyLanguageByCode('de');
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 7, 'found correct count (%TagList%)');

    $Search{'Parameters'}->{'String'} = '%XXXX%';
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 0, 'found nothing (%XXXX%)');

    $Search{'Parameters'}->{'String'} = '%TagList%';
    $Search{'PageSize'} = 2;
    $Search{'Page'} = 2;
    $Search = DE_EPAGES::Search::API::Object::PagerSearch->new( \%Search );
    is ($Search->countItems, 7, 'found correct count (%TagList%)');
    my $hPager = $Search->page->{'Pager'};
    is( $hPager->{'FirstPage'}, 1, 'Pager.FirstPage' );
    is( $hPager->{'LastPage'}, 4, 'Pager.LastPage' );
    is( $hPager->{'Page'}, 2, 'Pager.Page' );
    is( $hPager->{'NextPage'}, 3, 'Pager.NextPage' );
    is( $hPager->{'ItemOffset'}, 2, 'Pager.ItemOffset' );
    is( $hPager->{'URL'}, '?ViewAction=View&ObjectID=1&PageSize=2', 'Pager.URL' );
    my $aPages = $hPager->{'Pages'};
    is( scalar @$aPages, 4, 'Pager.Pages' );
    is( $hPager->{'URL'}, '?ViewAction=View&ObjectID=1&PageSize=2', 'Pager.URL' );
    my $aItems = $hPager->{'Items'};
    is( scalar @$aItems, 2, 'Pager.Items' );
    is( $aItems->[0]->{Object}->alias, 'Test3', 'Pager.Items[0]' );
    is( $aItems->[1]->{Object}->alias, 'Test4', 'Pager.Items[1]' );
    my $aAllItems = $Search->allItems;
    is( scalar @$aAllItems, 7, 'allItems' );
    my $aAllItemIDs = $Search->allItemIDs;
    is( scalar @$aAllItemIDs, 7, 'allItemIDs' );
    $aItems = $Search->items( 5 );
    is( scalar @$aItems, 0, 'items(5) non-existing page' );
    $aItems = $Search->items( 4 );
    is( scalar @$aItems, 1, 'items(4) last page' );
    $aItems = $Search->items( 3 );
    is( scalar @$aItems, 2, 'items(3)' );
    is( $aItems->[0]->alias, 'Test5', 'items(3)->[0]' );
    is( $aItems->[1]->alias, 'Test6', 'items(3)->[1]' );
    $aItems = $Search->items( 2 );
    is( scalar @$aItems, 2, 'items(2)' );
    $aItems = $Search->items( 1 );
    is( scalar @$aItems, 2, 'items(1)' );
    $aItems = $Search->itemIDs( 0 );
    is( scalar @$aItems, 7, 'itemIDs(0)' );
    $aItems = $Search->items( 0 );
    is( scalar @$aItems, 7, 'items(0)' );
}

sub cleanup {
    DeleteObjects('import.xml', $ROOTPATH );
}

sub init {
    ImportObjects('import.xml', $ROOTPATH );
}

__PACKAGE__->runScript;

1;
